import logging
from typing import Dict, Any, List, Optional
from telegram import Update
from telegram.ext import ContextTypes
from .base import InfoCommandsBase
from src.config.config import Config
from src.utils.token_display_formatter import get_formatter

logger = logging.getLogger(__name__)

class MarketCommands(InfoCommandsBase):
    """Handles all market-related commands."""

    def __init__(self, trading_engine, notification_manager):
        super().__init__(trading_engine, notification_manager)
        self.formatter = get_formatter()

    async def market_command(self, update: Update, context: ContextTypes.DEFAULT_TYPE) -> None:
        """Handle the /market command to show market information."""
        try:
            if not self._is_authorized(update):
                await self._reply(update, "❌ Unauthorized access.")
                return

            # Get token from command arguments or use default
            token = context.args[0].upper() if context.args else Config.DEFAULT_TRADING_TOKEN
            
            # Get market data
            market_data = await self.trading_engine.get_market_data(token)
            if not market_data:
                await self._reply(update, f"❌ Could not get market data for {token}")
                return
            
            # Format the message
            message = f"📊 <b>Market Information for {token}</b>\n\n"
            
            # Add price information
            message += "💰 <b>Price Information:</b>\n"
            message += f"Last Price: {self.formatter.format_price_with_symbol(market_data.get('last', 0))}\n"
            message += f"24h High: {self.formatter.format_price_with_symbol(market_data.get('high', 0))}\n"
            message += f"24h Low: {self.formatter.format_price_with_symbol(market_data.get('low', 0))}\n"
            message += f"24h Change: {market_data.get('change', 0):.2f}%\n"
            
            # Add volume information
            message += f"\n📈 <b>Volume Information:</b>\n"
            message += f"24h Volume: {self.formatter.format_amount(market_data.get('volume', 0))}\n"
            message += f"24h Turnover: {self.formatter.format_price_with_symbol(market_data.get('turnover', 0))}\n"
            
            # Add order book information if available
            if 'orderbook' in market_data:
                orderbook = market_data['orderbook']
                message += f"\n📚 <b>Order Book:</b>\n"
                message += f"Bid: {self.formatter.format_price_with_symbol(orderbook.get('bid', 0))}\n"
                message += f"Ask: {self.formatter.format_price_with_symbol(orderbook.get('ask', 0))}\n"
                message += f"Spread: {self.formatter.format_price_with_symbol(orderbook.get('spread', 0))}\n"
            
            await self._reply(update, message.strip())
            
        except Exception as e:
            logger.error(f"Error in market command: {e}")
            await self._reply(update, "❌ Error getting market information. Please try again later.")
